"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useBarcodeScanner = void 0;
var _react = require("react");
var _reactNativeVisionCamera = require("react-native-vision-camera");
var _LINKING_ERROR = require("../constants/LINKING_ERROR");
/**
 * Create a new instance of the {@link BarcodeScannerPlugin}.
 *
 * @param options - The plugin options {@link BarcodeScannerPluginOptions}.
 *
 * @returns The plugin instance {@link BarcodeScannerPlugin}.
 *
 * @throws Will throw an error if the plugin cannot be initialized.
 */
const createBarcodeScannerPlugin = options => {
  const plugin = _reactNativeVisionCamera.VisionCameraProxy.initFrameProcessorPlugin('barcodeScanner', {
    ...options
  });
  if (!plugin) throw new Error(_LINKING_ERROR.LINKING_ERROR);
  return {
    /**
     * Scans barcodes from a given frame.
     *
     * @param frame - The frame to process {@link Frame}.
     * @returns An array of recognized barcode data {@link Barcode}.
     */
    barcodeScanner: frame => {
      'worklet';

      return plugin.call(frame);
    }
  };
};

/**
 * Custom hook to use an instance of the {@link BarcodeScannerPlugin}.
 *
 * @param options - The plugin options {@link BarcodeScannerPluginOptions}.
 *
 * @returns {BarcodeScannerPlugin} A memoized plugin instance that will be
 * destroyed once the component using `useBarcodeScanner()` unmounts.
 *
 * @example
 * ```ts
 * const barcodePlugin = useBarcodeScanner({ formats: ['QR_CODE'] });
 * const frameProcessor = useFrameProcessor((frame) => {
 *   'worklet';
 *   const barcodes = barcodePlugin.barcodeScanner(frame);
 *   console.log(barcodes);
 * }, [barcodePlugin]);
 * ```
 */
const useBarcodeScanner = options => {
  return (0, _react.useMemo)(() => createBarcodeScannerPlugin(options), [options]);
};
exports.useBarcodeScanner = useBarcodeScanner;
//# sourceMappingURL=useBarcodeScanner.js.map