"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useObjectDetector = void 0;
var _react = require("react");
var _reactNativeVisionCamera = require("react-native-vision-camera");
var _LINKING_ERROR = require("../constants/LINKING_ERROR");
/**
 * Create a new instance of the {@link ObjectDetectorPlugin}.
 *
 * @param options - The plugin options {@link ObjectDetectorPluginOptions}.
 *
 * @returns The plugin instance {@link ObjectDetectorPlugin}.
 *
 * @throws Will throw an error if the plugin cannot be initialized.
 */
const createObjectDetectorPlugin = options => {
  const plugin = _reactNativeVisionCamera.VisionCameraProxy.initFrameProcessorPlugin('objectDetector', {
    ...options
  });
  if (!plugin) throw new Error(_LINKING_ERROR.LINKING_ERROR);
  return {
    /**
     * Detects objects in a given frame.
     *
     * @param frame - The frame processor {@link Frame}.
     * @returns An array containing the detected {@link Object} data.
     */
    objectDetector: frame => {
      'worklet';

      return plugin.call(frame);
    }
  };
};

/**
 * Custom hook to use an instance of the {@link ObjectDetectorPlugin}.
 *
 * @param options - The plugin options {@link ObjectDetectorPluginOptions}.
 *
 * @returns {ObjectDetectorPlugin} A memoized plugin instance that will be
 * destroyed once the component using `useObjectDetector()` unmounts.
 *
 * @example
 * ```ts
 * const objectPlugin = useObjectDetector({ enableMultipleObjects: true });
 * const frameProcessor = useFrameProcessor((frame) => {
 *   'worklet';
 *   const objects = objectPlugin.objectDetector(frame);
 *   console.log(objects);
 * }, [objectPlugin]);
 * ```
 */
const useObjectDetector = options => {
  return (0, _react.useMemo)(() => createObjectDetectorPlugin(options), [options]);
};
exports.useObjectDetector = useObjectDetector;
//# sourceMappingURL=useObjectDetector.js.map