import { useMemo } from 'react';
import { VisionCameraProxy } from 'react-native-vision-camera';
import { LINKING_ERROR } from '../constants/LINKING_ERROR';

/**
 * Create a new instance of the {@link BarcodeScannerPlugin}.
 *
 * @param options - The plugin options {@link BarcodeScannerPluginOptions}.
 *
 * @returns The plugin instance {@link BarcodeScannerPlugin}.
 *
 * @throws Will throw an error if the plugin cannot be initialized.
 */
const createBarcodeScannerPlugin = options => {
  const plugin = VisionCameraProxy.initFrameProcessorPlugin('barcodeScanner', {
    ...options
  });
  if (!plugin) throw new Error(LINKING_ERROR);
  return {
    /**
     * Scans barcodes from a given frame.
     *
     * @param frame - The frame to process {@link Frame}.
     * @returns An array of recognized barcode data {@link Barcode}.
     */
    barcodeScanner: frame => {
      'worklet';

      return plugin.call(frame);
    }
  };
};

/**
 * Custom hook to use an instance of the {@link BarcodeScannerPlugin}.
 *
 * @param options - The plugin options {@link BarcodeScannerPluginOptions}.
 *
 * @returns {BarcodeScannerPlugin} A memoized plugin instance that will be
 * destroyed once the component using `useBarcodeScanner()` unmounts.
 *
 * @example
 * ```ts
 * const barcodePlugin = useBarcodeScanner({ formats: ['QR_CODE'] });
 * const frameProcessor = useFrameProcessor((frame) => {
 *   'worklet';
 *   const barcodes = barcodePlugin.barcodeScanner(frame);
 *   console.log(barcodes);
 * }, [barcodePlugin]);
 * ```
 */
export const useBarcodeScanner = options => {
  return useMemo(() => createBarcodeScannerPlugin(options), [options]);
};
//# sourceMappingURL=useBarcodeScanner.js.map